/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co. KG                 *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 2014 - 2017  SEGGER Microcontroller GmbH & Co. KG        *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : main.c
Purpose : Generic application start
*/

#include <stdio.h>
#include <stdlib.h>

#ifndef COM
  #include "com.h"
  #define COM
#endif

#ifndef GLOBAL
  #include "global.h"
  #define GLOBAL
#endif

#ifndef STM32F4
  #include <stm32f4xx.h>
  #define STM32F4
#endif

#ifndef STDBOOL
  #include <stdbool.h>
  #define STDBOOL
#endif

#ifndef UI
  #include "ui.h"
  #define UI
#endif

#ifndef INITS
  #include "inits.h"
  #define INITS
#endif

/*********************************************************************
*
*       main()
*
*  Function description
*   Application entry point.
*/

enum testmode{
  NO_TEST = 0,
  SINGLE_TEST = 1,
  PERMANENT_TEST = 2
}Testmode;

void TIM6_DAC_IRQHandler(){
  //Reset interrupt flag
  TIM6 -> SR = ~TIM_SR_UIF;

  //Check buttons and refresh leds
  uiCheckAndRefresh();
}

void USART3_IRQHandler(){
  //Reset interrupt flag
  //COM -> SR &= ~USART_SR_UIF;

  //Check usart-status-register
  int c = 1;
}

void main(){
  Testmode = NO_TEST;
  //initClock();
  initNVIC();
  initPins();
  initTimer();
  initCOM();

  //Softwareinit
  uiInit();

  TIM6 -> CR1 |= TIM_CR1_CEN;
  COM -> CR1 |= USART_CR1_UE;
  //COM -> CR1 |= USART_CR1_PEIE;
  uiSetLED(LED_FATALERT, LED_BLINK1x);
  char c = 0;
  //Test Pin Init
  while(true){
    if(uiButtonWasTapped(BUTTON1)){
      Testmode ++;
      switch(Testmode){
        case NO_TEST:
          uiSetLED(LED_TEST, LED_DARK);
          break;

        case SINGLE_TEST:
          uiSetLED(LED_TEST, LED_SHINE);
          break;

        case PERMANENT_TEST:
          uiSetLED(LED_TEST, LED_BLINK2x);
          break;
        
        default:
          Testmode = NO_TEST;
          uiSetLED(LED_TEST, LED_DARK);
      }
    }
    switch(Testmode){
      case NO_TEST:
        break;
      
      case SINGLE_TEST:
        if(uiButtonWasTapped(BUTTON2)){
          COM -> DR = 0b01010101;
        }
        break;
      
      case  PERMANENT_TEST:
        COM -> DR = 0b01010101;
    }

    /*//Test for led state
    if(uiButtonWasTapped(BUTTON1)){
      uiSetLED(LED_THROTTON, LED_DARK);
    }
    if(uiButtonWasTapped(BUTTON2)){
      uiSetLED(LED_THROTTON, LED_BLINK1x);
    }
    if(uiButtonWasTapped(BUTTON3)){
      uiSetLED(LED_THROTTON, LED_BLINK2x);
    }
    if(uiButtonWasTapped(BUTTON4)){
      uiSetLED(LED_THROTTON, LED_BLINK4x);
    }
    if(uiButtonWasTapped(BUTTON5)){
      uiSetLED(LED_THROTTON, LED_SHINE);
    }*/
    
  }
}

/*************************** End of file ****************************/
