#ifndef STM32F4
  #include <stm32f4xx.h>
  #define STM32F4
#endif

#ifndef GLOBAL
  #include "global.h"
  #define GLOBAL
#endif

#ifndef STDBOOL
  #include <stdbool.h>
  #define STDBOOL
#endif

#ifndef INITS
  #include "inits.h"
  #define INITS
#endif

#ifndef UI
  #include "ui.h"
  #define UI
#endif

//This contains user interface control

//Constants for ui control
const int uiButtonCycleTime = 20;     //Time period for button check
const int uiMaxCycleTime = 40000;     //Max time cycle
const int uiLED1xBlinktime = 500;     //1x blinking period
const int uiLED2xBlinktime = 250;     //2x blinking period
const int uiLED4xBlinktime = 125;     //4x blinking period
const int uiButtonWaitForHold = 300;  //Waittime until pressed button is detected as held

//Internal variables used by ui-control-functions
int uiCycleTimeState = 0;

//Memoryflags for internal flip-flop-function
bool button1pressed;
bool button2pressed;
bool button3pressed;
bool button4pressed;
bool button5pressed;

//Count time if buttons are hold for longer time
int button1holdtime;
int button2holdtime;
int button3holdtime;
int button4holdtime;
int button5holdtime;

//Memoryflags for pressed buttons for asynchronous actcions
bool tmpbutton1pressed;
bool tmpbutton2pressed;
bool tmpbutton3pressed;
bool tmpbutton4pressed;
bool tmpbutton5pressed;

//LED states memory
enum uiLEDState LEDSTATE_FATALERT;
enum uiLEDState LEDSTATE_FATERR;
enum uiLEDState LEDSTATE_ERR;
enum uiLEDState LEDSTATE_TEST;
enum uiLEDState LEDSTATE_RACE;
enum uiLEDState LEDSTATE_FUELON;
enum uiLEDState LEDSTATE_THROTTON;

//Buffer for changed LED states
bool LEDCHANGEDSTATE_FATALERT;
bool LEDCHANGEDSTATE_FATERR;
bool LEDCHANGEDSTATE_ERR;
bool LEDCHANGEDSTATE_TEST;
bool LEDCHANGEDSTATE_RACE;
bool LEDCHANGEDSTATE_FUELON;
bool LEDCHANGEDSTATE_THROTTON;

//Init all user interface variables
void uiInit(){
  //Button states
  button1pressed = false;
  button2pressed = false;
  button3pressed = false;
  button4pressed = false;
  button5pressed = false;
  tmpbutton1pressed = false;
  tmpbutton2pressed = false;
  tmpbutton3pressed = false;
  tmpbutton4pressed = false;
  tmpbutton5pressed = false;

  button1holdtime = 0;
  button2holdtime = 0;
  button3holdtime = 0;
  button4holdtime = 0;
  button5holdtime = 0;

  LEDSTATE_FATALERT = LED_DARK;
  LEDSTATE_FATERR = LED_DARK;
  LEDSTATE_ERR = LED_DARK;
  LEDSTATE_TEST = LED_DARK;
  LEDSTATE_RACE = LED_DARK;
  LEDSTATE_FUELON = LED_DARK;
  LEDSTATE_THROTTON = LED_DARK;
}

//Check buttons and LEDs
void uiCheckAndRefresh(){
  uiCycleTimeState++;
  if(uiCycleTimeState >= uiMaxCycleTime){uiCycleTimeState = 0;}

  //Check buttons if button cycle time is over
  if(uiCycleTimeState % uiButtonCycleTime == 0){
    //Check Button1
    if(BUTTON1_PORT -> IDR & GPIO_IDR_ID5){
      button1pressed = true;
    }
    if(!(BUTTON1_PORT -> IDR & GPIO_IDR_ID5) & button1pressed){
      button1pressed = false;
      //Set actionfla for Button1
      tmpbutton1pressed = true;
    }

    //Check BUTTON2
    if(BUTTON2_PORT -> IDR & GPIO_IDR_ID6){
      button2pressed = true;
    }
    if(!(BUTTON2_PORT -> IDR & GPIO_IDR_ID6) & button2pressed){
      button2pressed = false;
      //Set actionfla for Button2
      tmpbutton2pressed = true;
    }

    //Check BUTTON3
    if(BUTTON3_PORT -> IDR & GPIO_IDR_ID7){
      button3pressed = true;
    }
    if(!(BUTTON3_PORT -> IDR & GPIO_IDR_ID7) & button3pressed){
      button3pressed = false;
      //Set actionfla for Button3
      tmpbutton3pressed = true;
    }

    //Check BUTTON4
    if(BUTTON4_PORT -> IDR & GPIO_IDR_ID12){
      button4pressed = true;
    }
    if(!(BUTTON4_PORT -> IDR & GPIO_IDR_ID12) & button4pressed){
      button4pressed = false;
      //Set actionfla for Button4
      tmpbutton4pressed = true;
    }

    //Check BUTTON5
    if(BUTTON5_PORT -> IDR & GPIO_IDR_ID13){
      button5pressed = true;
    }
    if(!(BUTTON5_PORT -> IDR & GPIO_IDR_ID13) & button5pressed){
      button5pressed = false;
      //Set actionfla for Button5
      tmpbutton5pressed = true;
    }
  }

  //Controll Fatal Alert LED state
  if(LEDCHANGEDSTATE_FATALERT){
    switch(LEDSTATE_FATALERT){
      case LED_DARK:
        LED_FATALERT_PORT -> BSRR = LED_FATALERT_OFF;
        LEDCHANGEDSTATE_FATALERT = false;
        break;

      case LED_SHINE:
        LED_FATALERT_PORT -> BSRR = LED_FATALERT_ON;
        LEDCHANGEDSTATE_FATALERT = false;
        break;

      case LED_BLINK1x:
        if(((uiCycleTimeState % uiLED1xBlinktime) == 0) & (uiCycleTimeState != 0)){
          if(!(LED_FATALERT_PORT -> ODR & LED_FATALERT_ODR)){
            LED_FATALERT_PORT -> BSRR = LED_FATALERT_OFF;
          }
          else{
            LED_FATALERT_PORT -> BSRR = LED_FATALERT_ON;
          }
        }        
        break;
      
      case LED_BLINK2x:
        if((uiCycleTimeState % uiLED2xBlinktime) == 0){
          if(!(LED_FATALERT_PORT -> ODR & LED_FATALERT_ODR)){
            LED_FATALERT_PORT -> BSRR = LED_FATALERT_OFF;
          }
          else{
            LED_FATALERT_PORT -> BSRR = LED_FATALERT_ON;
          }
        }        
        break;
      
      case LED_BLINK4x:
        if((uiCycleTimeState % uiLED4xBlinktime) == 0){
          if(!(LED_FATALERT_PORT -> ODR & LED_FATALERT_ODR)){
            LED_FATALERT_PORT -> BSRR = LED_FATALERT_OFF;
          }
          else{
            LED_FATALERT_PORT -> BSRR = LED_FATALERT_ON;
          }
        }        
    }
  }

  //Controll Fatal Error LED state
  if(LEDCHANGEDSTATE_FATERR){
    switch(LEDSTATE_FATERR){
      case LED_DARK:
        LED_FATERR_PORT -> BSRR = LED_FATERR_OFF;
        LEDCHANGEDSTATE_FATERR = false;
        break;

      case LED_SHINE:
        LED_FATERR_PORT -> BSRR = LED_FATERR_ON;
        LEDCHANGEDSTATE_FATERR = false;
        break;

      case LED_BLINK1x:
        if(((uiCycleTimeState % uiLED1xBlinktime) == 0) & (uiCycleTimeState != 0)){
          if(!(LED_FATERR_PORT -> ODR & LED_FATERR_ODR)){
            LED_FATERR_PORT -> BSRR = LED_FATERR_OFF;
          }
          else{
            LED_FATERR_PORT -> BSRR = LED_FATERR_ON;
          }
        }        
        break;
      
      case LED_BLINK2x:
        if((uiCycleTimeState % uiLED2xBlinktime) == 0){
          if(!(LED_FATERR_PORT -> ODR & LED_FATERR_ODR)){
            LED_FATERR_PORT -> BSRR = LED_FATERR_OFF;
          }
          else{
            LED_FATERR_PORT -> BSRR = LED_FATERR_ON;
          }
        }        
        break;
      
      case LED_BLINK4x:
        if((uiCycleTimeState % uiLED4xBlinktime) == 0){
          if(!(LED_FATERR_PORT -> ODR & LED_FATERR_ODR)){
            LED_FATERR_PORT -> BSRR = LED_FATERR_OFF;
          }
          else{
            LED_FATERR_PORT -> BSRR = LED_FATERR_ON;
          }
        }        
    }
  }

  //Controll Error LED state
  if(LEDCHANGEDSTATE_ERR){
    switch(LEDSTATE_ERR){
      case LED_DARK:
        LED_ERR_PORT -> BSRR = LED_ERR_OFF;
        LEDCHANGEDSTATE_ERR = false;
        break;

      case LED_SHINE:
        LED_ERR_PORT -> BSRR = LED_ERR_ON;
        LEDCHANGEDSTATE_ERR = false;
        break;

      case LED_BLINK1x:
        if(((uiCycleTimeState % uiLED1xBlinktime) == 0) & (uiCycleTimeState != 0)){
          if(!(LED_ERR_PORT -> ODR & LED_ERR_ODR)){
            LED_ERR_PORT -> BSRR = LED_ERR_OFF;
          }
          else{
            LED_ERR_PORT -> BSRR = LED_ERR_ON;
          }
        }        
        break;
      
      case LED_BLINK2x:
        if((uiCycleTimeState % uiLED2xBlinktime) == 0){
          if(!(LED_ERR_PORT -> ODR & LED_ERR_ODR)){
            LED_ERR_PORT -> BSRR = LED_ERR_OFF;
          }
          else{
            LED_ERR_PORT -> BSRR = LED_ERR_ON;
          }
        }        
        break;
      
      case LED_BLINK4x:
        if((uiCycleTimeState % uiLED4xBlinktime) == 0){
          if(!(LED_ERR_PORT -> ODR & LED_ERR_ODR)){
            LED_ERR_PORT -> BSRR = LED_ERR_OFF;
          }
          else{
            LED_ERR_PORT -> BSRR = LED_ERR_ON;
          }
        }        
    }
  }

  //Controll Test LED state
  if(LEDCHANGEDSTATE_TEST){
    switch(LEDSTATE_TEST){
      case LED_DARK:
        LED_TEST_PORT -> BSRR = LED_TEST_OFF;
        LEDCHANGEDSTATE_TEST = false;
        break;

      case LED_SHINE:
        LED_TEST_PORT -> BSRR = LED_TEST_ON;
        LEDCHANGEDSTATE_TEST = false;
        break;

      case LED_BLINK1x:
        if(((uiCycleTimeState % uiLED1xBlinktime) == 0) & (uiCycleTimeState != 0)){
          if(!(LED_TEST_PORT -> ODR & LED_TEST_ODR)){
            LED_TEST_PORT -> BSRR = LED_TEST_OFF;
          }
          else{
            LED_TEST_PORT -> BSRR = LED_TEST_ON;
          }
        }        
        break;
      
      case LED_BLINK2x:
        if((uiCycleTimeState % uiLED2xBlinktime) == 0){
          if(!(LED_TEST_PORT -> ODR & LED_TEST_ODR)){
            LED_TEST_PORT -> BSRR = LED_TEST_OFF;
          }
          else{
            LED_TEST_PORT -> BSRR = LED_TEST_ON;
          }
        }        
        break;
      
      case LED_BLINK4x:
        if((uiCycleTimeState % uiLED4xBlinktime) == 0){
          if(!(LED_TEST_PORT -> ODR & LED_TEST_ODR)){
            LED_TEST_PORT -> BSRR = LED_TEST_OFF;
          }
          else{
            LED_TEST_PORT -> BSRR = LED_TEST_ON;
          }
        }        
    }
  }

  //Controll race LED state
  if(LEDCHANGEDSTATE_RACE){
    switch(LEDSTATE_RACE){
      case LED_DARK:
        LED_RACE_PORT -> BSRR = LED_RACE_OFF;
        LEDCHANGEDSTATE_RACE = false;
        break;

      case LED_SHINE:
        LED_RACE_PORT -> BSRR = LED_RACE_ON;
        LEDCHANGEDSTATE_RACE = false;
        break;

      case LED_BLINK1x:
        if(((uiCycleTimeState % uiLED1xBlinktime) == 0) & (uiCycleTimeState != 0)){
          if(!(LED_RACE_PORT -> ODR & LED_RACE_ODR)){
            LED_RACE_PORT -> BSRR = LED_RACE_OFF;
          }
          else{
            LED_RACE_PORT -> BSRR = LED_RACE_ON;
          }
        }        
        break;
      
      case LED_BLINK2x:
        if((uiCycleTimeState % uiLED2xBlinktime) == 0){
          if(!(LED_RACE_PORT -> ODR & LED_RACE_ODR)){
            LED_RACE_PORT -> BSRR = LED_RACE_OFF;
          }
          else{
            LED_RACE_PORT -> BSRR = LED_RACE_ON;
          }
        }        
        break;
      
      case LED_BLINK4x:
        if((uiCycleTimeState % uiLED4xBlinktime) == 0){
          if(!(LED_RACE_PORT -> ODR & LED_RACE_ODR)){
            LED_RACE_PORT -> BSRR = LED_RACE_OFF;
          }
          else{
            LED_RACE_PORT -> BSRR = LED_RACE_ON;
          }
        }        
    }
  }

  //Controll fuel on LED state
  if(LEDCHANGEDSTATE_FUELON){
    switch(LEDSTATE_FUELON){
      case LED_DARK:
        LED_FUELON_PORT -> BSRR = LED_FUELON_OFF;
        LEDCHANGEDSTATE_FUELON = false;
        break;

      case LED_SHINE:
        LED_FUELON_PORT -> BSRR = LED_FUELON_ON;
        LEDCHANGEDSTATE_FUELON = false;
        break;

      case LED_BLINK1x:
        if(((uiCycleTimeState % uiLED1xBlinktime) == 0) & (uiCycleTimeState != 0)){
          if(!(LED_FUELON_PORT -> ODR & LED_FUELON_ODR)){
            LED_FUELON_PORT -> BSRR = LED_FUELON_OFF;
          }
          else{
            LED_FUELON_PORT -> BSRR = LED_FUELON_ON;
          }
        }        
        break;
      
      case LED_BLINK2x:
        if((uiCycleTimeState % uiLED2xBlinktime) == 0){
          if(!(LED_FUELON_PORT -> ODR & LED_FUELON_ODR)){
            LED_FUELON_PORT -> BSRR = LED_FUELON_OFF;
          }
          else{
            LED_FUELON_PORT -> BSRR = LED_FUELON_ON;
          }
        }        
        break;
      
      case LED_BLINK4x:
        if((uiCycleTimeState % uiLED4xBlinktime) == 0){
          if(!(LED_FUELON_PORT -> ODR & LED_FUELON_ODR)){
            LED_FUELON_PORT -> BSRR = LED_FUELON_OFF;
          }
          else{
            LED_FUELON_PORT -> BSRR = LED_FUELON_ON;
          }
        }        
    }
  }

  //Controll throttle on LED state
  if(LEDCHANGEDSTATE_THROTTON){
    switch(LEDSTATE_THROTTON){
      case LED_DARK:
        LED_THROTTON_PORT -> BSRR = LED_THROTTON_OFF;
        LEDCHANGEDSTATE_THROTTON = false;
        break;

      case LED_SHINE:
        LED_THROTTON_PORT -> BSRR = LED_THROTTON_ON;
        LEDCHANGEDSTATE_THROTTON = false;
        break;

      case LED_BLINK1x:
        if(((uiCycleTimeState % uiLED1xBlinktime) == 0) & (uiCycleTimeState != 0)){
          if(!(LED_THROTTON_PORT -> ODR & LED_THROTTON_ODR)){
            LED_THROTTON_PORT -> BSRR = LED_THROTTON_OFF;
          }
          else{
            LED_THROTTON_PORT -> BSRR = LED_THROTTON_ON;
          }
        }        
        break;
      
      case LED_BLINK2x:
        if((uiCycleTimeState % uiLED2xBlinktime) == 0){
          if(!(LED_THROTTON_PORT -> ODR & LED_THROTTON_ODR)){
            LED_THROTTON_PORT -> BSRR = LED_THROTTON_OFF;
          }
          else{
            LED_THROTTON_PORT -> BSRR = LED_THROTTON_ON;
          }
        }        
        break;
      
      case LED_BLINK4x:
        if((uiCycleTimeState % uiLED4xBlinktime) == 0){
          if(!(LED_THROTTON_PORT -> ODR & LED_THROTTON_ODR)){
            LED_THROTTON_PORT -> BSRR = LED_THROTTON_OFF;
          }
          else{
            LED_THROTTON_PORT -> BSRR = LED_THROTTON_ON;
          }
        }        
    }
  }
}

bool uiButtonWasTapped(enum uiButton b){
  bool tmp;
  switch(b){
    case BUTTON1:
      tmp = tmpbutton1pressed;
      tmpbutton1pressed = false;
      return tmp;

    case BUTTON2:
      tmp = tmpbutton2pressed;
      tmpbutton2pressed = false;
      return tmp;

    case BUTTON3:
      tmp = tmpbutton3pressed;
      tmpbutton3pressed = false;
      return tmp;

    case BUTTON4:
      tmp = tmpbutton4pressed;
      tmpbutton4pressed = false;
      return tmp;
      
    case BUTTON5:
      tmp = tmpbutton5pressed;
      tmpbutton5pressed = false;
      return tmp;
  }
}

bool uiButtonIsHeld(enum uiButton b){
  return false;
  switch(b){
    case BUTTON1:
      return false;
    case BUTTON2:
      return false;
    case BUTTON3:
      return false;
    case BUTTON4:
      return false;
    case BUTTON5:
      return false;
  }
}

void uiSetLED(enum uiLED led, enum uiLEDState state){
  switch(led){
    case LED_FATERR:
      LEDSTATE_FATERR = state;
      LEDCHANGEDSTATE_FATERR = true;
      break;

    case LED_FATALERT:
      LEDSTATE_FATALERT = state;
      LEDCHANGEDSTATE_FATALERT = true;
      break;

    case LED_ERR:
      LEDSTATE_ERR = state;
      LEDCHANGEDSTATE_ERR = true;
      break;

    case LED_TEST:
      LEDSTATE_TEST = state;
      LEDCHANGEDSTATE_TEST = true;
      break;

    case LED_RACE:
      LEDSTATE_RACE = state;
      LEDCHANGEDSTATE_RACE = true;
      break;

    case LED_FUELON:
      LEDSTATE_FUELON = state;
      LEDCHANGEDSTATE_FUELON = true;
      break;

    case LED_THROTTON:
      LEDSTATE_THROTTON = state;
      LEDCHANGEDSTATE_THROTTON = true;
      break;
  }
}

enum uiLEDState uiGetLED(enum uiLED led){
  switch(led){
    case LED_FATERR:
      return LEDSTATE_FATERR;

    case LED_FATALERT:
      return LEDSTATE_FATALERT;

    case LED_ERR:
      return LEDSTATE_ERR;

    case LED_TEST:
      return LEDSTATE_TEST;

    case LED_RACE:
      return LEDSTATE_RACE;

    case LED_FUELON:
      return LEDSTATE_FUELON;

    case LED_THROTTON:
      return LEDSTATE_THROTTON;
  }
}